#!/usr/bin/env node
const commander = require('commander')
const readline = require('readline')
const path = require('path')
const pkg = require(path.resolve(__dirname, '../package.json'))
const fs = require('fs')

function tokenFn(val){
  return val.split(':')
}

commander
  .name('Sgs-node')
  .version(pkg.version)
  .description(pkg.description)
  .usage('[options] <params ...>')
  .option('init', '初始化')
  .option('-f, --force', '强制覆盖当前文件夹')
  .option('build', 'webpack构建')
  .option('-a, --all', '全部构建')
  .option('-d, --dll', '构建DLL')
  .option('-c, --client', '构建Client')
  .option('-s, --server', '构建Server')
  .option('-p, --prepack', '构建Prepack')
  .option('create', '创建页面')
  .option('-f, --folder <folder>', '创建 [folder] 页面')
  .option('dev', '运行 Dev')
  .option('tools', '执行工具')
  .option('-t, --type <type>', '执行[type]工具')
  .option('--token <username>:<password>', '用户名', tokenFn)
  .on('--help', () => {
    console.log('  更多信息请访问')
    console.log('        http://sa.sogou.com/')
  })
  .parse(process.argv)

if (commander.init) {
  const createApplication = require('../webpack/create-app.js')

  const destinationPath = commander.args.shift() || '.'

  const appname = createAppName(path.resolve(destinationPath)) || 'hello-world'

  emptyDirectory(destinationPath, function(empty) {
    if (empty || commander.force) {
      console.log('Issue')
      createApplication(appname, destinationPath)
    } else {
      confirm('destination is not empty, continue? [y/N] ', function(ok) {
        if (ok) {
          process.stdin.destroy()
          createApplication(appname, destinationPath)
        } else {
          console.error('aborting')
          process.exit()
        }
      })
    }
  })
}

function emptyDirectory(dir, callback) {
  fs.readdir(dir, function(err, files) {
    if (err && err.code !== 'ENOENT') throw err
    callback(!files || !files.length)
  })
}

function confirm(msg, callback) {
  var rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  })

  rl.question(msg, function(input) {
    rl.close()
    callback(/^y|yes|ok|true$/i.test(input))
  })
}

function createAppName(pathName) {
  return path
    .basename(pathName)
    .replace(/[^A-Za-z0-9.-]+/g, '-')
    .replace(/^[-_.]+|-+$/g, '')
    .toLowerCase()
}

if (commander.create) {
  if (commander.folder) {
    const createPage = require('../webpack/create-page.js')

    ;(async () => {
      createPage(commander.folder)
    })()
  }
}

if (commander.build) {
  let build
  if (commander.all) {
    console.log('打包 ALL')
    build = require('../webpack/build-all.js')
  } else if (commander.dll) {
    console.log('打包 DLL')
    build = require('../webpack/build-dll.js')
  } else if (commander.client) {
    console.log('打包 Client')
    build = require('../webpack/build-client.js')
  } else if (commander.server) {
    console.log('打包 Server')
    build = require('../webpack/build-server.js')
  } else if (commander.prepack) {
    console.log('生成 Prepack')
    build = require('../webpack/prepack.js')
  } else {
    console.log('打包 ALL')
    build = require('../webpack/build-all.js')
  }

  ;(async () => {
    try {
      build()
    } catch (e) {
      console.log(e)
    }
  })()
}

if (commander.dev) {
  const update = require('./../webpack/update')
  const prepack = require('./../webpack/hook-before-webpack')

  ;(async () => {
    try {
      await update()

      await prepack()
      const {
        checkDllVersion,
        createDllVersion
      } = require('./../webpack/dllVersion.js')

      const isNeedRunDll = checkDllVersion()

      if (isNeedRunDll) {
        const build = require('../webpack/build-dll.js')
        await build()
        createDllVersion()
      }

      require(process.cwd() + '/server')
    } catch (e) {
      console.log(e)
    }
  })()
}

if (commander.tools) {
  if (commander.type) {
    let type = commander.type,
      token = commander.token || [];

    let [username, password] = token

    if (!['ued'].includes(type)) {
      console.log(`[Sgs-node Update]: ${type} not found`)
      return
    }
    const update = require(`./../tools/update-${type}.js`)

    ;(async () => {
      await update(username, password)
    })()
  }
}
