/**
 * Created by hinotohui on 18/11/20.
 */
const { app: appConfig, feed_detail: kvConfig } = require('../config')
const { LoadBalance } = require('../loadBalance/default')
const logger = require('../sgs_loggger')
const http = require('http')

let kvServers = require('../factory/serverFactory')['create'](kvConfig, {
  maxSockets: appConfig.kv_limit,
  maxFreeSockets: appConfig.kv_limit,
  timeout: appConfig.kv_keepalive_timeout,
  freeSocketTimeout: appConfig.kv_keepalive_timeout
})

// 调用函数注意需要try catch
function request (path, id, server, method = 'GET', timeOut = appConfig.kv_timeout) {
  return Promise.race([
    new Promise((resolve, reject) => {
      setTimeout(() => {
        reject(new Error('server ' + id + ' timeout'))
      }, timeOut)
    }),

    new Promise((resolve, reject) => {
      let req = http.request({
        'host': server.host,
        'port': server.port,
        'path': path,
        'method': method,
        'agent': server.agent
      }, (res) => {
        res.setEncoding(appConfig.kv_charset || 'utf8')
        let chunks = []
        res.on('data', (chunk) => {
          chunks.push(chunk)
        })
        res.on('end', () => { resolve(chunks.join('')) })
      })

      req.on('error', (err) => {
        reject(new Error('server ' + id + ' request error'))
      })

      req.end()
    })
  ]).then((data) => {
    return data
  }, (e) => {
    throw e
  })
}

// 此处特意暴露异常，调用函数注意需要try catch
async function getResult (path, retryTimes = 2, method = 'GET', servers = kvServers) {
  let result = null
  let excludes = []

  for (let i = 0; i < retryTimes; i++) {
    try {
      let [id, server] = LoadBalance.nextServer(servers, excludes)
      excludes.push(id)
      result = await request(path, id, server, method, appConfig.kv_timeout)
      break
    } catch (e) {
      logger.error(e)
    }
  }

  return result
}

process.on('exit', () => {
  kvServers && kvServers.forEach((v) => {
    let agent = v['agent']

    agent.destroy && agent.destroy()
  })
})

module.exports = {
  getResult
}
