const { redis: redisConfig } = require('../config')
const logger = require('../sgs_loggger')
const { circuitBreaker } = require('../backend/circuitBreaker')
const redis = require('redis')

let client = redis.createClient({
  'host': redisConfig['fe_cache']['host'],
  'port': redisConfig['fe_cache']['port'],
  'password': redisConfig['fe_cache']['password'],
  'db': redisConfig['fe_cache']['db']
})

client.on('error', (err) => {
  logger.error(err)
})

async function getValue (key) {
  try {
    return await circuitBreaker(50, new Promise((resolve, reject) => {
      if (!key) { resolve(null) } else {
        client.get(key, (err, value) => {
          if (!err) {
            resolve(value)
          } else {
            logger.error(err)
            resolve(null)
          }
        })
      }
    }))
  } catch (e) {
    return null
  }
}
function setValue (key, value, expire = 600) {
  client.set(key, value, 'EX', expire)
}

function quit () {
  client.quit()
}

process.on('exit', () => {
  client.quit()
})

module.exports = {
  getValue,
  setValue
  // quit
}
