const fs = require('fs-extra')
const path = require('path')
const glob = require('glob')
const co = require('co')

module.exports = ({
  enableDefaultActions = true,
  actionsDir: customActionsDir
}) => {
  const defaultActionsDir = path.resolve(__dirname, '../actions')
  let dirs = [customActionsDir, defaultActionsDir]

  if (!enableDefaultActions) {
    dirs = [customActionsDir]
  }

  const getActions = dir => {
    return glob.sync(`${dir}/*.js`).map(file => {
      return file.replace(`${dir}/`, '').replace('.js', '')
    })
  }

  const actionKeys = dirs.map(getActions).reduce((prev, cur) => {
    return Array.from(new Set(prev.concat(cur)))
  }, [])

  return async (ctx, next) => {
    let reqPath = ctx.path
    const actionKey = reqPath.split('/')[1]
    if (!actionKeys.includes(actionKey)) {
      return await next()
    }

    let match = reqPath.replace(`/${actionKey}`, '')

    let api

    dirs.some(dir => {
      let targetFile = path.join(dir, `${actionKey}.js`)

      let actions
      if (!fs.existsSync(targetFile)) return false

      actions = require(targetFile)

      if (typeof actions === 'object' && actions[match]) {
        api = actions[match]
      } else if (typeof actions === 'function') {
        api = actions
      }

      return api
    })

    if (!api) {
      return await next()
    }

    let funcType = api && api.constructor.name

    if (funcType === 'GeneratorFunction') {
      await co.call(ctx, api.call(ctx))
    } else {
      await api.call(ctx)
    }
  }
}
