'use strict'

const url = require('url')
const proxyKoa = require('koa-proxy')
const convert = require('koa-convert')
const pathToRegExp = require('path-to-regexp')
const logger = require('../utils/logger')

const middleware = ({ apiConfig }) => {
  let paths = apiConfig && Object.keys(apiConfig)
  let pathRegExp = paths.map(v => pathToRegExp(v))

  return async (ctx, next) => {
    let reqPath = ctx.path
    if (
      ['/weball/api', '/feed/api'].every(v => {
        return !reqPath.startsWith(v)
      })
    ) {
      return await next()
    }

    if (!paths || paths.length === 0) {
      return await next()
    }

    let map = defaultMapFunc
    let index = pathRegExp.findIndex(re => {
      return re.exec(reqPath)
    })

    if (index < 0) {
      return await next()
    }

    let c = paths[index]
    let host = apiConfig[c].url
    if (typeof apiConfig[c].map === 'function') {
      map = apiConfig[c].map
    }

    // 打印转发
    logProxyRule(host, reqPath, map)
    // 将 proxy 返回的 generator 转化为 async
    let fn = convert(
      proxyKoa({
        host,
        map
      })
    )

    return await fn(ctx, next)
  }
}

function defaultMapFunc(path) {
  return path
}

function logProxyRule(host, path, map) {
  let targetUrl = map(path)
  targetUrl = url.resolve(host, targetUrl)
  logger.proxy(`${path} => ${targetUrl}`)
}

module.exports = middleware
