const { Monitor } = require('./../utils/monitor/Monitor.js')
const root = process.cwd()
const path = require('path')
const fse = require('fs-extra')
const config = require('config')
const pathConfig = config.get('path')
const errorHtml =
  fse.readFileSync(path.resolve(pathConfig.static, './error.html'), {
    encoding: 'utf8'
  }) || '服务器错误'

const RuntimeException = require('../errors/exceptions/runtime.exception')
// const InvalidParamException = require('../errors/exceptions/invalid-param.exception')
// const InvalidRequestException = require('../errors/exceptions/invalid-request.exception')
// const InvalidResponseException = require('../errors/exceptions/invalid-response.exception')
// const InvalidRouteException = require('../errors/exceptions/invalid-route.exception')
// const NoresultException = require('../errors/exceptions/noresult.exception')
// const ServiceException = require('../errors/exceptions/service.exception')

module.exports = config => {
  return async (ctx, next) => {
    try {
      ctx.Monitor = new Monitor('cost', ctx)
      ctx.Monitor.benchmark('start', 'total')

      await next()

      ctx.Monitor.benchmark('end', 'total')
      ctx.Monitor.send()
    } catch (e) {
      if (e instanceof RuntimeException) {
        let noResultMonitor = new Monitor('noresult', ctx)
        noResultMonitor.assign({
          label: e.name,
          errorCode: 1,
          errorMsg: e.stack
        })

        noResultMonitor.send()
      } else {
        let noResultMonitor = new Monitor('noresult', ctx)
        noResultMonitor.assign({
          label: '程序运行异常',
          errorCode: 1,
          errorMsg: e.stack
        })
        noResultMonitor.send()

        let errorMonitor = new Monitor('error', ctx)
        errorMonitor.assign({
          name: e.name,
          message: e.message,
          stacktrace: e.stack
        })
        errorMonitor.send()
      }

      // logger.error(e, [ctx.request.url])
      // if (ctx.status === 404) {
      //   ctx.status = 500
      // }
      
      ctx.status = 200

      let msg = (e && e.toString()) || '服务器错误'
      if (
        ctx.accept.headers.accept &&
        ~ctx.accept.headers.accept.indexOf('json')
      ) {
        ctx.body = { code: -1, msg: msg, data: null }
      } else {
        ctx.body = errorHtml
      }
    }
  }
}
