const root = process.cwd()
const path = require('path')
const fs = require('fs')

let defaultConfig = {
  folder: path.resolve(root, 'mock'),
  prefix: '/mock',
  nameIndex: 2
}

function camelCase (str) {
  return str.replace(/\/([a-zA-Z])/g, w => {
    return w.substring(1, 2).toUpperCase() + w.substring(2)
  })
}

module.exports = (config = {}, app) => {
  let mockConfig = Object.assign({}, defaultConfig, config)
  let { folder: mockPath, prefix, nameIndex } = mockConfig

  return async (ctx, next) => {
    let reqPath = ctx.path
    if (!reqPath.startsWith(prefix)) {
      return await next()
    }

    let pathArr = reqPath.split('/').slice(nameIndex + 1)

    // 第一位为文件名
    let filename = pathArr[0]
    let methodName = pathArr.slice(1).join('/')
    methodName = camelCase(methodName)
    let filePath = path.join(mockPath, `output/${filename}.js`)
    if (!fs.existsSync(filePath)) {
      return (ctx.status = 404)
    }

    delete require.cache[filePath]
    let methods = require(filePath)
    let api

    if ((api = methods[methodName])) {
      let res = await api({ query: ctx.query, body: ctx.request.body })

      ctx.body = {
        code: 0,
        msg: '',
        data: res
      }
    } else {
      // 方法不存在
      return (ctx.status = 404)
    }
  }
}
