const os = require('os')

let ip = ''
const networkInterfaces = os.networkInterfaces()

;(function () {
  Object.keys(networkInterfaces).forEach(item => {
    if (item.indexOf('lo') == -1) {
      networkInterfaces[item].forEach(item => {
        if (item['family'].toLowerCase() == 'ipv4') {
          ip = item['address']
        }
      })
    }
  })
})()

const hostname = os.hostname()

class Monitor {
  constructor (type, ctx) {
    this.type = type
    this.ip = ip
    this.hostname = hostname
    this.timestamp = Date.now()
    this.index = (
      (process && process.env && process.env['NODE_APP_INSTANCE']) ||
      0
    ).toString()
    this.url = ctx.url
    this.msg = {}
  }

  setMsg (msg) {
    this.msg = msg || {}
  }

  /**
   *
   * @param {*} k
   * @param {*} v
   */
  add (k, v) {
    this.msg[k] = v
  }
  
  /**
   *
   * @param {*} v
   */
  assign (v) {
    this.msg = Object.assign(this.msg || {}, v || {})
  }

  benchmark (action, type) {
    if (action == 'start') {
      this.msg[type] = Date.now()
    } else if (action == 'end') {
      this.msg[type] = Date.now() - (this.msg[type] || 0)
    }
  }

  send () {
    if (process.send) {
      process.send({
        type: 'pm2-monitor',
        data: {
          type: this.type,
          content: JSON.stringify(this)
        }
      })
    } else {
      console.log(JSON.stringify(this))
    }
  }
}

/*
let monitor=new Monitor('cost',{url:'/baiu'})
monitor.benchmark('start','total')
monitor.benchmark('end','total')
monitor.add('name','Error')
monitor.send() */

module.exports = {
  Monitor
}
