const { execSync } = require('child_process')
const root = process.cwd()
const path = require('path')
const fs = require('fs')
const fse = require('fs-extra')
const glob = require('glob')

function update (username = '', password = '') {
  let configPath = path.resolve(root, './ued.config.js')
  let exists = fs.existsSync(configPath)
  if (!exists) {
    console.warn(`[Svn Config]: Config not found`)

    return
  }
  const config = require(configPath)
  if (!config.url) {
    console.warn('[Svn Warn]: Url not found')
    return
  }

  let svnPath = path.resolve(root, config.local || './.temp/svn')
  let copyPaths = config.copyPaths || []

  let _convert_ =
    config.convert ||
    (str => {
      return str
    })
  let svnUri = _convert_(config.url)
  let cmd = `svn checkout ${svnUri} ${svnPath}`
  let token = ''
  if (username) {
    token += ` --username ${username}`
    if (password) {
      token += ` --password ${password}`
    }
  }

  cmd = cmd + token

  try {
    let updateStr = execSync(cmd).toString('utf-8')
    console.log(`[Svn Tip]: Svn update is over, ${updateStr}`)

    if (copyPaths.length === 0) {
      console.warn(`[Svn Copy]: Copypaths not found`)
    } else {
      for (const val of copyPaths) {
        let [src, dest] = val
        console.log(`[Svn Copy]: ${src}`)
        fse.copySync(path.resolve(svnPath, src), dest)
      }

      console.log(`[Svn Copy]: Copy Over`)
    }

    if (!config.regexp || !config.pattern || !config.glob) {
      console.log(`[Svn Replace]: Noneed Replace`)
    } else {
      let encode = config.encode || 'utf-8'
      let files = glob.sync(path.resolve(root, config.glob))
      for (const file of files) {
        let filepath = path.resolve(root, file)
        console.log(`[Svn Replace]: ${filepath}`)

        let content = fse.readFileSync(filepath, encode)
        let str = content.replace(config.regexp, config.pattern)
        fse.writeFileSync(filepath, str, encode)
      }
      console.log(`[Svn Replace]: Replace Over`)
    }

    console.log(`[Svn Over]: Svn update is over`)
  } catch (e) {
    console.error(`[Svn Error]: ${e}`)
  }
}

module.exports = update
