const pathConfig = require('config').get('path')
const projectConfig = require('config').get('project')
const path = require('path')
const glob = require('glob')
const fs = require('fs')
const fse = require('fs-extra')
const Mustache = require('mustache')

const {
  pathConfig: { template: tempatePath }
} = require('./config.js')

function createPage (folder) {
  if (typeof folder !== 'string') {
    return console.warn(`[create-page]: ${folder} must be string`)
  }

  if (fs.existsSync(path.resolve(pathConfig.pages, folder))) {
    return console.warn(`[create-page]: ${folder} is exists`)
  }

  try {
    glob
      .sync(tempatePath + '/page/**')
      .filter(v => {
        let stat = fs.statSync(v)
        return !stat.isDirectory()
      })
      .forEach(v => {
        let filepath = v.slice((tempatePath + '/page').length)
        let targetfile = path.join(
          pathConfig.pages,
          `./${folder}`,
          `./${filepath}`
        )
        if (filepath === '/config.js') {
          let config = fse.readFileSync(v, 'utf-8')
          let content = Mustache.render(config, {
            project: projectConfig.name,
            folder,
            html: projectConfig.html
          })
          fse.outputFile(targetfile, content, 'utf-8')
        } else {
          fse.ensureFileSync(targetfile)
          fse.copyFileSync(v, targetfile)
        }
      })
    console.log(`创建完成：「${folder}」，请愉快的 CODING 吧`)
  } catch (e) {
    console.error(`创建异常 ${folder}：`)
    console.error(e)
  }
}

module.exports = createPage
