const pathConfig = require('config').get('path')
const webpackConf = require('config').get('webpack')

const path = require('path')
const webpack = require('webpack')
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin')
const { VueLoaderPlugin } = require('vue-loader')
const VueSSRClientPlugin = require('vue-server-renderer/client-plugin')

const { alias } = require('./../alias')
const { publicPath } = require('./../output.js')
const { clientEntry } = require('./../entry.js')
const { entries: assetsEntry = {} } = webpackConf

module.exports = {
  target: 'web',
  mode: 'development',
  entry: Object.assign(
    {},
    assetsEntry,
    { global: pathConfig.global },
    clientEntry
  ),
  output: {
    filename: 'js/[name].js',
    path: pathConfig.dist,
    publicPath: publicPath
  },
  resolve: {
    alias,
    extensions: ['.js', '.json', '.vue']
  },
  module: {
    rules: [
      {
        test: /\.(png|jpe?g|gif|svg|woff2?|eot|ttf|otf)(\?.*)?$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              limit: 10000,
              name: 'image/[name].[hash].[ext]'
            }
          }
        ]
      },
      {
        test: /\.js$/,
        // exclude: /(node_modules)/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              presets: [
                [
                  '@babel/preset-env',
                  {
                    targets: {
                      browsers: ['Android >= 4.0', 'ios >= 6']
                    },
                    debug: true,
                    include: [],
                    useBuiltIns: false
                  }
                ]
              ],
              plugins: ['@babel/plugin-syntax-dynamic-import']
            }
          }
        ]
      },
      {
        test: /\.vue$/,
        use: [
          {
            loader: 'vue-loader',
            options: {
              loaders: {
                css: ['vue-style-loader', 'css-loader'],
                scss: ['vue-style-loader', 'css-loader', 'sass-loader'],
                sass: ['vue-style-loader', 'css-loader', 'sass-loader'],
                js: {
                  loader: 'babel-loader',
                  // exclude: /(node_modules)/,
                  options: {
                    presets: [
                      [
                        '@babel/preset-env',
                        {
                          targets: {
                            browsers: ['Android >= 4.0', 'ios >= 6']
                          },
                          debug: true,
                          include: [],
                          useBuiltIns: false
                        }
                      ]
                    ],
                    plugins: ['@babel/plugin-syntax-dynamic-import']
                  }
                }
              }
            }
          }
        ]
      },
      {
        test: /\.css$/,
        use: ['vue-style-loader', 'css-loader']
      },
      {
        test: /\.scss$/,
        use: ['vue-style-loader', 'css-loader', 'sass-loader']
      },
      {
        test: /\.sass$/,
        use: ['vue-style-loader', 'css-loader', 'sass-loader']
      }
    ]
  },
  performance: {
    maxEntrypointSize: 300000,
    hints: false
  },
  plugins: [
    new VueLoaderPlugin(),
    new FriendlyErrorsPlugin(),
    new webpack.EnvironmentPlugin(['NODE_ENV']),
    new webpack.DefinePlugin({
      'process.env.VUE_ENV': '"client"'
    }),
    new webpack.DllReferencePlugin({
      manifest: require(`${pathConfig.dll}/vendor.json`)
    }),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
    new webpack.NamedModulesPlugin(),
    new VueSSRClientPlugin()
  ]
}
