const tasksConf = require('config').get('tasks')
const Copy = require('copy')
const pathConfig = require('config').get('path')
const { printRoute } = require('./router.js')

const _move_ = (from, to) => {
  return new Promise((resolve, reject) => {
    Copy(from, to, (err, files) => {
      if (err) reject(err)
      else resolve(files)
    })
  })
}

async function move() {
  const patterns = [
    {
      from: `${pathConfig.dist}/css/*.css`,
      to: pathConfig.static + '/css'
    },
    {
      from: `${pathConfig.dist}/js/*.js`,
      to: pathConfig.static + '/js'
    },
    {
      from: `${pathConfig.dist}/js/*.js.map`,
      to: pathConfig.static + '/js'
    },
    {
      from: `${pathConfig.dist}/image/*.*`,
      to: pathConfig.static + '/image'
    },
    {
      from: `${pathConfig.dll}/css/*.css`,
      to: pathConfig.static + '/css'
    },
    {
      from: `${pathConfig.dll}/js/*.js`,
      to: pathConfig.static + '/js'
    },
    {
      from: `${pathConfig.dll}/image/*.*`,
      to: pathConfig.static + '/image'
    },
    {
      from: pathConfig.favicon,
      to: pathConfig.static
    }
  ]

  try {
    for (const { from, to } of patterns) {
      await _move_(from, to)
    }

    await printRoute()
  } catch (err) {
    console.error(err)
  }
}

const afterHook = async function() {
  let { afterHook } = tasksConf

  let hooks = afterHook.filter(v => {
    return typeof v === 'function'
  })

  for (const index in hooks) {
    if (hooks.hasOwnProperty(index)) {
      const fn = hooks[index]

      await fn.call()
    }
  }
}

async function hook() {
  await move()

  await afterHook()
}

module.exports = hook
