const webpack = require('webpack')
const SWPrecachePlugin = require('sw-precache-webpack-plugin')
const VueSSRClientPlugin = require('vue-server-renderer/client-plugin')
const VueLoaderPlugin = require('vue-loader/lib/plugin')
const CleanWebpackPlugin = require('clean-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const ParallelUglifyPlugin = require('webpack-parallel-uglify-plugin')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const extractCSS = new ExtractTextPlugin({
  filename: 'css/[name].css',
  allChunks: true
})

const serverConfig = require('config').get('server')
const pathConfig = require('config').get('path')

const { isDevelopment, isProduction } = serverConfig

const dllEntry = require('./dllEntry.js')

const plugins = [
  new VueLoaderPlugin(),
  new VueSSRClientPlugin(),
  new webpack.ProgressPlugin(),
  new webpack.EnvironmentPlugin(['NODE_ENV']),
  new webpack.DefinePlugin({
    'process.env.NODE_ENV': '"production"',
    'process.env.buildTime': JSON.stringify(Date.now())
  }),
  new webpack.ProvidePlugin({
    qs: 'query-string'
  }),
  new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
  new webpack.optimize.ModuleConcatenationPlugin(),
  ...Object.keys(dllEntry).map(
    v =>
      new webpack.DllReferencePlugin({
        manifest: require(`${pathConfig.dll}/${v}.json`)
      })
  )
  // extractCSS,
  // new CleanWebpackPlugin(
  //   ['dist/**/*.(js|css|json)', 'static/**/*.(js|css)'], // glob 匹配
  //   {
  //     root: pathConfig.root, // 根目录
  //     exclude: ['favicon.ico'], // 不包含
  //     verbose: true, // 开启输出
  //     dry: false // 启动删除文件
  //   }
  // )
]

if (!isDevelopment) {
  plugins.push(
    ...[
      new webpack.HotModuleReplacementPlugin(),
      new webpack.NoEmitOnErrorsPlugin(),
      new webpack.NamedModulesPlugin()
    ]
  )
} else {
  plugins.push(
    ...[
      extractCSS,
      new CleanWebpackPlugin(
        ['dist/**/*.(js|css|json)', 'static/**/*.(js|css)'], // glob 匹配
        {
          root: pathConfig.root, // 根目录
          exclude: ['favicon.ico'], // 不包含
          verbose: true, // 开启输出
          dry: false // 启动删除文件
        }
      )
    ]
  )
}

if (isProduction) {
  plugins.push(
    new ParallelUglifyPlugin({
      uglifyJS: {
        compress: {
          warnings: false
        }
      }
    })
  )
}

module.exports = plugins
