const {
  pathConfig: { template }
} = require('./config')

const glob = require('glob')
const path = require('path')
const fs = require('fs')
const fse = require('fs-extra')
const Mustache = require('mustache')
const { pages, prepackPath, src } = require('config').get('path')

const appServerTemplatePath = path.resolve(template, 'ssr/app-server.js')
const appTemplatePath = path.resolve(template, 'ssr/app.js')
const clientTemplatePath = path.resolve(template, 'ssr/entry-client.js')
const serverTemplatePath = path.resolve(template, 'ssr/entry-server.js')

const createSSRFile = async (key = 'index') => {
  const reader = p => fs.readFileSync(p, { encoding: 'utf-8' })

  const appContent = Mustache.render(reader(appTemplatePath), {
    app: path.resolve(pages, `${key}`),
    global: path.resolve(src, 'global.js')
  })

  const appServerContent = Mustache.render(reader(appServerTemplatePath), {
    app: path.resolve(pages, `${key}`),
    global: path.resolve(src, 'global-server.js')
  })

  const serverContent = reader(serverTemplatePath)
  const clientContent = reader(clientTemplatePath)

  let appPath = path.resolve(prepackPath, `./ssr/${key}/app.js`)
  let clientPath = path.resolve(prepackPath, `./ssr/${key}/entry-client.js`)
  let appPathServer = path.resolve(prepackPath, `./ssr/${key}/app-server.js`)
  let serverPath = path.resolve(prepackPath, `./ssr/${key}/entry-server.js`)

  const writer = (p, c) => fse.outputFileSync(p, c, { encoding: 'utf-8' })

  let entries = [
    [appPathServer, appServerContent],
    [appPath, appContent],
    [serverPath, serverContent],
    [clientPath, clientContent]
  ]

  for (const entry of entries) {
    await writer.apply(null, entry)
  }
}

const prepack = async () => {
  let keys = glob.sync(path.resolve(pages, './**/app.vue')).map(v => {
    return v.split('/').slice(-2, -1)[0]
  })

  for (const key of keys) {
    await createSSRFile(key)
  }
}

module.exports = prepack
