const path = require('path')
const packageJson = require('package-json')
const semver = require('semver')

async function getPkgVersion (name) {
  try {
    return await packageJson(name)
  } catch (e) {
    console.log(e)
  }
}

async function update () {
  const { version: currentVersion, name } = require(path.resolve(
    __dirname,
    '../package.json'
  ))

  const { version: latestVersion } = await getPkgVersion(name)

  let isNeedUpdate = semver.lt(currentVersion, latestVersion)

  return { name, currentVersion, latestVersion, isNeedUpdate }
}

function getWidthOfWindow () {
  if (typeof process === 'object' && process.stdout && process.stdout.columns) {
    return process.stdout.columns
  }

  return 80
}

async function print () {
  const { name, currentVersion, latestVersion, isNeedUpdate } = update()

  if (isNeedUpdate) {
    let text = [
      `Update avaliable ${currentVersion} -> ${latestVersion}`,
      `Run npm update ${name}@${latestVersion} to update`
    ]
    const maxWidth = getWidthOfWindow()
    const box = {
      topLeft: '╭',
      topRight: '╮',
      bottomRight: '╯',
      bottomLeft: '╰',
      vertical: '│',
      horizontal: '─'
    }

    const lines = []
    let line = box.topLeft + box.horizontal.repeat(maxWidth - 2) + box.topRight
    let lineOfCenter = ''
    let lineOfLast =
      box.bottomLeft + box.horizontal.repeat(maxWidth - 2) + box.bottomRight

    lines.push(line)

    function paddingWithCode (str, len = 0, code = ' ') {
      return code.repeat(len) + str + code.repeat(len)
    }

    text.forEach(v => {
      let width = Math.trunc((maxWidth - v.length - 2) / 2)
      let str = paddingWithCode(v, width)
      if (str.length % 2 == 1) {
        str += ' '
      }
      lineOfCenter = box.vertical + str + box.vertical
      lines.push(lineOfCenter)
    })

    lines.push(lineOfLast)

    console.log(lines.join('\n'))
  }
}

module.exports = print
