const pathConfig = require('config').get('path')
const path = require('path')
const webpack = require('webpack')
const VueLoaderPlugin = require('vue-loader/lib/plugin')
const VueSSRServerPlugin = require('vue-server-renderer/server-plugin')
const nodeExternals = require('webpack-node-externals')
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// const extractCSS = require('./extract.js')
const { alias } = require('./alias')
const { serverEntry } = require('./entry')

const { publicPath } = require('./output.js')

const webpackConfig = {
  mode: 'production',
  target: 'node',
  entry: Object.assign({ global: pathConfig.globalServer }, serverEntry),
  output: {
    filename: 'js/[name]-server.js',
    path: pathConfig.dist,
    publicPath: publicPath,
    libraryTarget: 'commonjs2'
  },
  resolve: {
    alias,
    extensions: ['.js', '.json', '.vue']
  },
  externals: nodeExternals({
    // do not externalize CSS files in case we need to import it from a dep
    whitelist: /\.css$/
  }),
  module: {
    rules: [
      {
        test: /\.vue$/,
        use: [
          {
            loader: 'vue-loader',
            options: {
              compilerOptions: {
                preserveWhitespace: false
              },
              presets: [['env', { modules: false }]],
              plugins: ['syntax-dynamic-import'],
              loaders: {
                css: ['vue-style-loader', 'css-loader'],
                sass: ['vue-style-loader', 'css-loader', 'sass-loader'],
                scss: ['vue-style-loader', 'css-loader', 'sass-loader'],
                js: {
                  loader: 'babel-loader',
                  options: {
                    presets: ['@babel/preset-env']
                  }
                }
              }
            }
          }
        ]
      },
      {
        test: /\.js$/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              presets: ['@babel/preset-env'],
              plugins: ['syntax-dynamic-import']
            }
          }
        ]
      },
      {
        test: /\.(png|jpe?g|gif|svg|woff2?|eot|ttf|otf)(\?.*)?$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              limit: 10000,
              name: 'image/[name].[hash].[ext]'
            }
          }
        ]
      },
      {
        test: /\.css$/,
        use: [MiniCssExtractPlugin.loader, 'css-loader']
      },
      {
        test: /\.scss$/,
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader']
      },
      {
        test: /\.sass$/,
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader']
      }
    ]
  },
  plugins: [
    new VueLoaderPlugin(),
    new MiniCssExtractPlugin({
      filename: 'css/[name].[contenthash].css'
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"production"',
      'process.env.buildTime': JSON.stringify(Date.now())
    }),
    new webpack.DllReferencePlugin({
      manifest: require(`${pathConfig.dll}/vendor.json`)
    }),
    new VueSSRServerPlugin(),
    new FriendlyErrorsPlugin()
  ],
  optimization: {
    splitChunks: {}
  }
}

module.exports = { webpackConfig }
